@@ -20,8 +20,8 @@
  * @author Sam Lang, Sam Meder
  *
  * $RCSfile: globus_gsi_proxy.c,v $
- * $Revision: 1.33.2.1 $
- * $Date: 2008/09/15 15:05:06 $
+ * $Revision: 1.35.4.1 $
+ * $Date: 2010/01/04 22:03:15 $
  */
 #endif
 
@@ -166,8 +166,6 @@ globus_l_gsi_proxy_deactivate(void)
 
     GLOBUS_I_GSI_PROXY_DEBUG_ENTER;
 
-    X509V3_EXT_cleanup();
-
     globus_module_deactivate(GLOBUS_OPENSSL_MODULE);
 
     globus_module_deactivate(GLOBUS_GSI_CREDENTIAL_MODULE);
@@ -457,7 +455,10 @@ globus_gsi_proxy_create_req(
         GLOBUS_I_GSI_PROXY_DEBUG_PRINT(3, "******  END PROXYCERTINFO  ******\n");
     }
     
-    if (!X509_REQ_sign(handle->req, handle->proxy_key, EVP_md5()))
+    if (!X509_REQ_sign(handle->req, handle->proxy_key,
+            handle->attrs->signing_algorithm
+            ? handle->attrs->signing_algorithm
+            : EVP_sha1()))
     {
         GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
             result,
@@ -1053,6 +1054,7 @@ globus_l_gsi_proxy_sign_key(
     X509 *                              issuer_cert = NULL;
     X509_EXTENSION *                    pci_ext = NULL;
     X509_EXTENSION *                    extension;
+    const EVP_MD *                      issuer_digest;
     int                                 position;
     EVP_PKEY *                          issuer_pkey = NULL;
     globus_result_t                     result = GLOBUS_SUCCESS;
@@ -1163,7 +1165,7 @@ globus_l_gsi_proxy_sign_key(
     
     if(pci_NID != NID_undef)
     {
-        EVP_MD *                        sha1 = EVP_sha1();
+        const EVP_MD *                  sha1 = EVP_sha1();
         unsigned char                   md[SHA_DIGEST_LENGTH];
         long                            sub_hash;
         unsigned int                    len;
@@ -1543,18 +1545,32 @@ globus_l_gsi_proxy_sign_key(
     /* right now if MD5 isn't requested as the signing algorithm,
      * we throw an error
      */
-    if(EVP_MD_type(handle->attrs->signing_algorithm) != NID_md5)
+    issuer_digest = EVP_get_digestbynid(
+            OBJ_obj2nid(issuer_cert->sig_alg->algorithm));
+    if (issuer_digest == NULL)
     {
         GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
             result,
             GLOBUS_GSI_PROXY_ERROR_WITH_HANDLE,
-            (_PCSL("The signing algorithm: %s is not currently allowed."
-             "\nUse MD5 to sign certificate requests"),
-             OBJ_nid2sn(EVP_MD_type(handle->attrs->signing_algorithm))));
+            (_PCSL("The signing algorithm used by the ceritificate cannot "
+            " be used to sign a proxy.")));
+        goto done;
+    }
+    if (handle->attrs->signing_algorithm != NULL && 
+        EVP_MD_type(handle->attrs->signing_algorithm) !=
+        EVP_MD_type(issuer_digest))
+    {
+        GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
+            result,
+            GLOBUS_GSI_PROXY_ERROR_WITH_HANDLE,
+            (_PCSL("The signing algorithm: %s does not match "
+            "issuer certificate algorithm %s."),
+             OBJ_nid2sn(EVP_MD_type(handle->attrs->signing_algorithm)),
+             OBJ_nid2sn(EVP_MD_type(issuer_digest))));
         goto done;
     }
     
-    if(!X509_sign(*signed_cert, issuer_pkey, handle->attrs->signing_algorithm))
+    if(!X509_sign(*signed_cert, issuer_pkey, issuer_digest))
     {
         GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
             result,
@@ -1672,6 +1688,39 @@ globus_gsi_proxy_create_signed(
         "globus_gsi_proxy_create_signed";
     GLOBUS_I_GSI_PROXY_DEBUG_ENTER;
 
+    result = globus_gsi_cred_get_cert(issuer, &issuer_cert);
+    if(result != GLOBUS_SUCCESS)
+    {
+        GLOBUS_GSI_PROXY_ERROR_CHAIN_RESULT(
+            result,
+            GLOBUS_GSI_PROXY_ERROR_WITH_HANDLE);
+        goto exit;
+    }
+
+    if (handle->attrs->signing_algorithm == NULL)
+    {
+        const EVP_MD *                  issuer_digest;
+
+        issuer_digest = EVP_get_digestbynid(
+                OBJ_obj2nid(issuer_cert->sig_alg->algorithm));
+        if (issuer_digest == NULL)
+        {
+            GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
+                result,
+                GLOBUS_GSI_PROXY_ERROR_WITH_BIO,
+                (_PCSL("Certificate's signing algorithm not supported by "
+                "OpenSSL")));
+            goto exit;
+        }
+        result = globus_gsi_proxy_handle_attrs_set_signing_algorithm(
+                handle->attrs,
+                issuer_digest);
+        if (result != GLOBUS_SUCCESS)
+        {
+            goto exit;
+        }
+    }
+
     rw_mem_bio = BIO_new(BIO_s_mem());
     if(!rw_mem_bio)
     {
@@ -1778,15 +1827,6 @@ globus_gsi_proxy_create_signed(
         goto exit;
     }
 
-    result = globus_gsi_cred_get_cert(issuer, &issuer_cert);
-    if(result != GLOBUS_SUCCESS)
-    {
-        GLOBUS_GSI_PROXY_ERROR_CHAIN_RESULT(
-            result,
-            GLOBUS_GSI_PROXY_ERROR_WITH_HANDLE);
-        goto exit;
-    }
-
     if(!i2d_X509_bio(rw_mem_bio, issuer_cert))
     {
         GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
